/**
 * Standard HTTP Request
 * @param {Object} _params
 * @description The following are valid options to pass through:
 * @param {int} _params.timeout  	: int Timeout request
 * @param {string} _params.type		: string GET/POST
 * @param {string} _params.format		: string json, etc.
 * @param {object/string} _params.data		: mixed The data to pass
 * @param {string} _params.url			: string The url source to call
 * @param {function} _params.failure		: funtion A function to execute when there is an XHR error
 * @param {function} _params.success		: function when successful
 * @param {string} _params.passthrough : Any passthrough params
 * @param {string} _params.headers     : Array of request headers
 */
exports.request = function(_params) {
	if(Ti.Network.online) {
		// Setup the xhr object
		var xhr = Ti.Network.createHTTPClient();
		xhr.validatesSecureCertificate = false;
		
		// Set the timeout or a default if one is not provided
		xhr.timeout = _params.timeout ? _params.timeout : 10000;
        //xhr.setAutoEncodeUrl(false);
        
		// For mobile web CORs
		if(Ti.Platform.osname === "mobileweb") {
			xhr.withCredentials = true;
		}



		/**
		 * When XHR request is loaded
		 * @param _data - The data that was retrieved form the service call
		 */
        xhr.onload = function(_data) {
            if (_data) {
                try {
                    _data = _params.format === "json" ? JSON.parse(this.responseText) : this.responseData;
                } catch(ex) {
                    Ti.API.debug("HTTP.js Exception caught parsing result");
                    Ti.API.debug(JSON.stringify(this));
                }

                if (xhr.getResponseHeaders) {
                    var headers = xhr.getResponseHeaders();
                }

                if (_params.success) {
                    if (_params.passthrough) {
                        _params.success(_data, headers, _params.passthrough);
                    } else {
                        _params.success(_data, headers);
                    }
                } else {
                    return _data;
                }
            }
        }; 



        /**
		 * Response is complete and all headers are available
		 * at this point (4 = 'Done').
		 * Persist JSESSIONID if in cookie
		 */
		xhr.onreadystatechange = function() {
			switch(xhr.readyState) {
				case 4:
					if (_params.persistSessionId) {
						Ti.App.Properties.setString('cookie', null);
						Ti.App.Properties.setString('CartCookie', null);

						var cookie = xhr.getResponseHeader("set-cookie");
						if (cookie) {
							Ti.App.Properties.setString('cookie', cookie);
						}

						Ti.API.info('----------------------COOKIE FROM NATIVE SERVICE----------------------' + cookie);

						if (cookie && cookie.indexOf(('JSESSIONID') !== -1)) {
							var idMarker = 'JSESSIONID=';
							var id;
							var index = cookie.indexOf(idMarker);
							var startCookie = index + idMarker.length;
							var endCookie = cookie.indexOf(';');
							if (endCookie !== -1) {
								var cookies = cookie.split(';');
								for (var i = 0; i < cookies.length; i++) {
									if (cookies[i].indexOf('JSESSIONID') > -1) {
										var marker = 'JSESSIONID=';
										var start = cookies[i].indexOf(marker) + marker.length;
										var end = cookies[i].length;
										var id = cookies[i].substring(start, end);
										//alert('FOUND ID IN FOR LOOP: ' + id);
									}
								}
							} else {//the only cookie so no ';'
								var length = cookie.length;
								id = cookie.substring(startCookie, length);
							}
							if (id) {
								Ti.App.Properties.setString('JSESSIONID', id);
							}
						}
						if (cookie && cookie.indexOf('CartCookie') !== -1) {
							
							var idMarker = 'CartCookie=';
							var id;
							var index = cookie.indexOf(idMarker);
							var startCookie = index + idMarker.length;
							var endCookie = cookie.indexOf(';');
							if (endCookie !== -1) {
								var cookies = cookie.split(';');
								for (var i = 0; i < cookies.length; i++) {
									if (cookies[i].indexOf('CartCookie') > -1) {
										var marker = 'CartCookie=';
										var start = cookies[i].indexOf(marker) + marker.length;
										var end = cookies[i].length;
										var id = cookies[i].substring(start, end);
										//alert('FOUND CART ID IN FOR LOOP: ' + id);
									}
								}
							} else {//the only cookie so no ';'
								var length = cookie.length;
								id = cookie.substring(startCookie, length);
							}
							Ti.App.Properties.setString('CartCookie', id);
						}
					}
			}
		};
                
                
		if(_params.ondatastream) {
			xhr.ondatastream = function(_event) {
				if(_params.ondatastream) {
					_params.ondatastream(_event.progress);
				}
			};
		}



		/**
		 * Error handling
		 * @param {Object} _event The callback object
		 */
		xhr.onerror = function(_event) {
			if(_params.failure) {
			    Ti.API.error(JSON.stringify(this));
				_params.failure(this.responseText);
			} else {
				Ti.API.error(JSON.stringify(this));
			}
		};

		// Open the remote connection
		_params.type = _params.type ? _params.type : "GET";
		_params.async = _params.async ? _params.async : true;

		xhr.open(_params.type, _params.url, _params.async);

		if(_params.headers) {
			for (var i = 0, j = _params.headers.length; i < j; i++) {
				xhr.setRequestHeader(_params.headers[i].name, _params.headers[i].value);
			}
		}

        // send the data
		if(_params.data) {
	
            Ti.API.info("-----JSON DATA IN REQUEST: " + JSON.stringify(_params.data));
            Ti.API.info("-----JSON DATA UNSTRINGIFIED: " + _params.data);
            //var data = '/"serviceName/":/"getStoreCMSTemplateData/":/"rest-parameters/":{/"atg-rest-depth/":/"3/",/"atg-rest-show-rest-paths/":/"false/",/"arg1/":"/HomeScreenTemplate/",/"arg2/":/"{}/"}';
			//var convertedData = (Ti.Platform.osname === "android") ? JSON.stringify(_params.data) : _params.data;
			xhr.send(_params.data);
			//xhr.send(data);
			//alert("data: " + data);
			//xhr.send(_params.data);
		} else {
			xhr.send();
		}
	} else {
		Ti.API.info("No internet connection");
		alert("We aren't able to reach our servers.  Please turn on data or wait until you have a stronger signal.");
	}
};